int analogPin = 0; // read from multiplexer using analog input 0
int strobePin = 2; // strobe is attached to digital pin 2
int resetPin = 3; // reset is attached to digital pin 3
int spectrumValue[7]; // to hold a2d values
int conspectrumValue[7];
int filterValue = 80;
int function = 0;

const int lightPin1 = 4;
const int bulbCount = 50;
const int defaultIntensity = 0xCC;
const int broadcastBulb = 63;

const int buttonPin = 5;
int val = 0;
int old_val = 0;

const int PIR = 6;
int calibrationTime = 1;
const int ledPin = 13;

unsigned long lastTrigger = 0;
int on = 10; // in minutes
int reset = 15; // in minutes
int off = 0; // variable to store on off state

unsigned long seconds;
unsigned long lastSecond;

int r = 0, g = 15, b = 0; // variables for color cycling music visualizer and rainbow functions

int bulbOn [34];
int randomNumber = 0;

void setup() {
  Serial.begin(9600);
  
  randomSeed(analogRead(0));

  pinMode (ledPin, OUTPUT);
  pinMode (PIR, INPUT);
  digitalWrite (PIR, LOW);
  Serial.print("calibrating sensor ");
  for(int i = 0; i < calibrationTime; i++){
    Serial.print(".");
    delay(1000);
  }

  Serial.println(" done");
  Serial.println("SENSOR ACTIVE");
  delay(50);

  pinMode(buttonPin, INPUT);
  digitalWrite (buttonPin, HIGH);

  pinMode(analogPin, INPUT);
  pinMode(strobePin, OUTPUT);
  pinMode(resetPin, OUTPUT);
  analogReference(DEFAULT);

  digitalWrite(resetPin, LOW);
  digitalWrite(strobePin, HIGH);
  pinMode(lightPin1, OUTPUT);
  digitalWrite(lightPin1, LOW);
  fillString(bulbColor(0,0,0));
  fillString(bulbColor(0,15,0));
  delay (500);
  fillString(bulbColor(0,0,0));
}

void loop() {
  val = digitalRead(buttonPin);
  if ((val == HIGH) && (old_val == LOW)) {
    changeMode(function + 1);
  }
  old_val = val;

  digitalWrite(resetPin, HIGH);
  digitalWrite(resetPin, LOW);


  for (int i = 0; i < 7; i++) // store mapped msgeq7 values in an array
  {
    digitalWrite(strobePin, LOW);
    delayMicroseconds(30); // to allow the output to settle
    spectrumValue [i] = analogRead(analogPin);
    digitalWrite(strobePin, HIGH);
  }

  switch (checkMovement()) {
  case 1: 
    {
      switch (function) {
      case 1: 
        {
          fillBulbs (0, 32, bulbColor (13, 13, 13));
          if (off == 1) {
            off = 0;
          }
        }
        break;

      case 2:
        {
          randomBulb();
          
        }
        break;

      case 3: 
        {
          vuMeter();
        } 
        break;

      case 4: 
        {
          rainbowVisualizer();
        } 
        break;

      case 5: 
        {
          rainbow();
        } 
        break;
        
      case 6:
      {
        musicVisualizer();
      }
      break;
      
      case 7: 
        {
          fillBulbs (0, bulbCount, bulbColor (0, 0, 0));
        }
        break;

      }
    }
    break;

  case 2: 
    {
      changeMode(1);
    }
    break;

  default: 
    {
      fillBulbs (0, bulbCount, bulbColor (0, 0, 0));
    }
  }
}


/*
 * Lighting Functions- Sound to light
 */
 
void musicVisualizer() {
  mapAudio();
  int w = conspectrumValue[2] + 1;
  int p = conspectrumValue[1];
  int x = conspectrumValue[0];
  fillBulbs(0,13, bulbColor(x,p,w));
  int y = conspectrumValue[4] + 1;
  int q = conspectrumValue[5];
  int z = conspectrumValue[6];
  fillBulbs(13, 19, bulbColor (z,q,y));
}

void vuMeter() {
  int RAW = 0;
  int lastRAW = 0;
  RAW = spectrumValue[1];
  RAW = constrain(RAW, filterValue, 1023);
  RAW = map (RAW, filterValue, 1023, 0, 10);
  fillBulbs (16 + RAW, (bulbCount - RAW + 13), bulbColor(0,0,0));
  fillBulbs(16, RAW, bulbColor(0,0,15));
  lastRAW = RAW;
  delay (10);
}

void rainbowVisualizer() {
  if (secTime() == 1) {
    if (r == 0 & g == 15 & b < 15) {
      b = b + 1;
    } 
    else if (r == 0 & g > 0 & b == 15) {
      g = g - 1;
    } 
    else if (r < 15 & g == 0 & b == 15) {
      r = r + 1;
    } 
    else if (r == 15 & g == 0 & b > 0) {
      b = b - 1;
    } 
    else if (r == 15 & g < 15 & b == 0) {
      g = g + 1;
    } 
    else if (r > 0 & g == 15 & b == 0) {
      r = r - 1;
    }
  }
  /*
  Serial.print (r);
   Serial.print ("  ");
   Serial.print (g);
   Serial.print ("  ");
   Serial.print (b);
   Serial.println ("  ");
   */
  int RAWr = 0;
  int RAWg = 0;
  int RAWb = 0;
  int RAW = 0;
  RAW = spectrumValue[2];
  RAW = constrain(RAW, filterValue, 1023);
  RAWr = map (RAW, filterValue, 1023, 1, r);
  RAWg = map (RAW, filterValue, 1023, 0, g);
  RAWb = map (RAW, filterValue, 1023, 1, b);
  fillBulbs (13, 19, bulbColor (RAWr, RAWg, RAWb));
}

void randomBulb() {
  mapAudio();
  if (secTime() == 1) {
    for (int i = 0; i < 6; i++) {
      randomNumber = random(0,34);
      bulbOn[randomNumber] = random(0, 3);
    }
    
    for (int i = 0; i < 34; i++) {
      Serial.print(bulbOn[i]);
    }
    Serial.println();
    
  }
  for (int i = 0; i < 34; i++) {
    if (bulbOn[i] == 1){
      setBulb(i, bulbColor(0,conspectrumValue[1],0));
    } else if (bulbOn[i] == 0) {
      setBulb(i, bulbColor(0,0,conspectrumValue[5]));
    } else if (bulbOn[i] == 2) {
      setBulb(i, bulbColor(conspectrumValue[3],0,0));
    }
  }
}


/*
 * Lighting Functions- Fun Effects
 */
 
void rainbow() {
  int secTime2 = secTime();
  if (secTime2 == 1) {
    if (r == 0 & g == 15 & b < 15) {
      b = b + 1;
    } 
    else if (r == 0 & g > 0 & b == 15) {
      g = g - 1;
    } 
    else if (r < 15 & g == 0 & b == 15) {
      r = r + 1;
    } 
    else if (r == 15 & g == 0 & b > 0) {
      b = b - 1;
    } 
    else if (r == 15 & g < 15 & b == 0) {
      g = g + 1;
    } 
    else if (r > 0 & g == 15 & b == 0) {
      r = r - 1;
    }
    /*
    Serial.print (r);
     Serial.print ("  ");
     Serial.print (g);
     Serial.print ("  ");
     Serial.print (b);
     Serial.println ("  ");
     */
    fillBulbs (0, 34, bulbColor (r, g, b));
  }
}


 /*
 * Functions to simplify and organize code
 */

int secTime () {
  seconds = millis() / 1000;
  if (lastSecond < seconds) {
    lastSecond = seconds;
    return 1;
  } 
  else {
    return 0;
  }
}

void changeMode(int functionChange) {
  digitalWrite (ledPin, LOW);
  if (functionChange != function) {
    function = functionChange;
    if (function >= 8) {
      function = 1;
    }
    Serial.print ("Function Number");
    Serial.println (function);
  }
}

int checkMovement() {
  if (digitalRead (PIR) == HIGH) {
    lastTrigger = millis();
    digitalWrite (ledPin, HIGH);
  } 
  else {
    digitalWrite (ledPin, LOW);
  }
  unsigned long timeDiff = (millis() - lastTrigger);
  if (timeDiff < on * 60000) {
    return 1;
  } 
  else if (timeDiff > reset * 60000) {
    return 2;
  } 
  else if (timeDiff > on * 60000 && timeDiff < reset * 60000) {
    return 3;
  }
}

void mapAudio() {
    for (int i = 0; i < 7; i++) {
    int RAW = spectrumValue[i];
    RAW = constrain(RAW, filterValue, 1023);
    RAW = map (RAW, filterValue, 1023, 0, 15);
    conspectrumValue [i] = RAW;
  }
}



/*
 * Signalling for the data line
 */

void writeStart() {
  digitalWrite(lightPin1, HIGH);
  delayMicroseconds(7);
}

void writeBit(int b)
{
  switch (b)
  {
  case 0:
    digitalWrite(lightPin1, LOW);
    delayMicroseconds(3);
    digitalWrite(lightPin1, HIGH);
    delayMicroseconds(17);
    break;

  case 1:
    digitalWrite(lightPin1, LOW);
    delayMicroseconds(17);
    digitalWrite(lightPin1, HIGH);
    delayMicroseconds(3);
    break;
  }
}

void writeStop()
{
  digitalWrite(lightPin1, LOW);
  delayMicroseconds(32);
}

/*
 * Functions to set an individual bulb
 */

int bulbColor(int r, int g, int b)
{
  return (r+(g<<4)+(b<<8));
}


void setBulb(int bulb, int intensity, int color)
{
  writeStart();

  for(int i=5; i>=0; i--)
    writeBit(bitRead(bulb, i));

  for(int i=7; i>=0; i--)
    writeBit(bitRead(intensity, i));

  for(int i=11; i>=0; i--)
    writeBit(bitRead(color, i));

  writeStop();
}


//If no inensity is specified, use the default
void setBulb(int bulb, int color)
{
  setBulb(bulb, defaultIntensity, color);
}

/*
 * Functions to set a series of bulbs or the entire string
 */


void fillBulbs(int start, int count, int intensity, int color)
{
  while(count--)
    setBulb(start++, intensity, color);
} 

//If intensity is not specified, use the default
void fillBulbs(int start, int count, int color)
{
  fillBulbs(start, count, defaultIntensity, color);
} 


//Fill the entire string
void fillString(int intensity, int color)
{
  fillBulbs(0, bulbCount, intensity, color);
} 


//Fill the entire string with the default intensity
void fillString(int color)
{
  fillBulbs(0, bulbCount, defaultIntensity, color);
}

